@@ -7,12 +7,12 @@ use Carp qw(croak);
 use Scalar::Util qw(blessed);
 use English qw(-no_match_vars);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 our $AUTOLOAD;
 
 my $maybe_set_subname = sub { $ARG[1]; };
 
-# Will be shipping with Perl 5.22
+# Supported on Perl 5.22+
 eval {
     require Sub::Util;
 
@@ -47,66 +47,61 @@ sub has {
 
     croak q{Attribute not defined} unless defined $attr and length $attr;
 
-    if (exists $param{'get'}) {
-        my ($method_name, $getter) = (q{}, $param{'get'});
+    my @patches;
 
-        if ($getter eq q{+}) {
-            $method_name = $attr;
-        }
-        elsif ($getter eq q{-}) {
-            $method_name = q{_}. $attr;
-        }
-        else {
-            $method_name = $getter;
-        }
+    for my $each_attr (@{ ref $attr eq 'ARRAY' ? $attr : [$attr] }) {
+        croak sprintf(q{Attribute "%s" invalid}, $each_attr)
+            unless $each_attr =~ m{^[a-zA-Z_]\w*$}x;
 
-        $self->_meta->{$method_name} = $maybe_set_subname->($method_name, sub {
+        my %patch;
+
+        # getter
+        $patch{_define_name_of_getter($each_attr, \%param)} = sub {
             my $self = shift;
             return $self->_stash->{$attr};
-        });
+        };
+
+        # setter
+        {
+            my $method_name = _define_name_of_setter($each_attr, \%param);
+
+            if (exists $param{'default'}) {
+                my $default_value = $param{'default'};
+
+                $patch{$method_name} = sub {
+                    my $self = shift;
+
+                    if (@ARG == 1) {
+                        $self->_stash->{$attr} = shift;
+                    }
+                    elsif (ref $default_value eq 'CODE') {
+                        $self->_stash->{$attr} = $maybe_set_subname->(
+                            'default_value',
+                            $default_value,
+                        )->($self);
+                    }
+                    else {
+                        $self->_stash->{$attr} = $default_value;
+                    }
+
+                    return $self;
+                };
+            }
+            else {
+                $patch{$method_name} = sub {
+                    my ($self, $value) = @ARG;
+                    $self->_stash->{$attr} = $value;
+                    return $self;
+                };
+            }
+        }
+
+        push @patches, \%patch;
     }
 
-    if (exists $param{'set'}) {
-        my ($method_name, $setter) = (q{}, $param{'set'});
-
-        if ($setter eq q{+}) {
-            $method_name = q{set_}. $attr;
-        }
-        elsif ($setter eq q{-}) {
-            $method_name = q{_set_}. $attr;
-        }
-        else {
-            $method_name = $setter;
-        }
-
-        if (exists $param{'default'}) {
-            my $default_value = $param{'default'};
-
-            $self->_meta->{$method_name} = $maybe_set_subname->($method_name, sub {
-                my $self = shift;
-
-                if (@ARG == 1) {
-                    $self->_stash->{$attr} = shift;
-                }
-                elsif (ref $default_value eq 'CODE') {
-                    $self->_stash->{$attr} = $maybe_set_subname->(
-                        'default_value',
-                        $default_value,
-                    )->($self);
-                }
-                else {
-                    $self->_stash->{$attr} = $default_value;
-                }
-
-                return $self;
-            });
-        }
-        else {
-            $self->_meta->{$method_name} = $maybe_set_subname->($method_name, sub {
-                my ($self, $value) = @ARG;
-                $self->_stash->{$attr} = $value;
-                return $self;
-            });
+    for my $each_patch (@patches) {
+        while (my ($method_name, $code) = each %{ $each_patch }) {
+            $self->_meta->{$method_name} = $maybe_set_subname->($method_name, $code);
         }
     }
 
@@ -122,6 +117,9 @@ sub method {
     croak q{First argument must be a non empty string}
         unless defined $method_name and length $method_name;
 
+    croak sprintf(q{First argument "%s" invalid}, $method_name)
+        unless $method_name =~ m{^[a-zA-Z_]\w*$}x;
+
     croak q{Second argument must be a code ref}
         unless ref($code) eq 'CODE';
 
@@ -192,15 +190,66 @@ sub AUTOLOAD {
     }
 }
 
-# private methods
 sub proto {
     return $ARG[0]->{'proto'};
 }
+
 sub set_proto {
     $ARG[0]->{'proto'} = $ARG[1];
     return $ARG[0];
 }
 
+# private methods
+sub _define_name_of_getter {
+    my ($attr, $param) = @ARG;
+
+    my $method_name;
+
+    if (exists $param->{'get'}) {
+        my $getter = $param->{'get'};
+
+        if ($getter eq q{+}) {
+            $method_name = $attr;
+        }
+        elsif ($getter eq q{-}) {
+            $method_name = q{_}. $attr;
+        }
+        else {
+            $method_name = $getter;
+        }
+    }
+    else {
+        $method_name = $attr;
+    }
+
+    return $method_name;
+}
+
+sub _define_name_of_setter {
+    my ($attr, $param) = @ARG;
+
+    my $method_name;
+
+    if (exists $param->{'set'}) {
+        my $setter = $param->{'set'};
+
+        if ($setter eq q{+}) {
+            $method_name = q{set_}. $attr;
+        }
+        elsif ($setter eq q{-}) {
+            $method_name = q{_set_}. $attr;
+        }
+        else {
+            $method_name = $setter;
+        }
+    }
+    else {
+        $method_name = q{set_}. $attr;
+    }
+
+    return $method_name;
+}
+
 sub _meta {
     return $ARG[0]->{'meta'};
 }
@@ -227,78 +276,104 @@ JIP::Object - A simple object system.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 SYNOPSIS
 
-    use Test::More;
     use JIP::Object;
 
     my $obj = JIP::Object->new;
-    ok $obj, 'got object';
 
-    # Public access to the "foo"
     $obj->has('foo', (get => '+', set => '+'));
-    is($obj->set_foo(42)->foo, 42);
+    $obj->set_foo(42)->foo; # 42
 
-    # Private access to the "bar"
-    $obj->has('bar', (get => '-', set => '-'));
-    is($obj->_set_bar(42)->_bar, 42);
-
-    # Create user-defined names for getters/setters
-    $obj->has('wtf' => (get => 'wtf_getter', set => 'wtf_setter'));
-    is($obj->wtf_setter(42)->wtf_getter, 42);
-
-    # Pass an optional first argument of setter to set
-    # a default value, it should be a constant or callback.
-    $obj->has('baz' => (get => '+', set => '+', default => 42));
-    is($self->set_baz->baz, 42);
-
-    $obj->has('qux' => (get => '+', set => '+', default => sub {
-        my $self = shift;
-        return $self->baz;
-    }));
-    is($self->set_qux->qux, 42);
-
-    # Define a new method
     $obj->method('say_foo', sub {
         my $self = shift;
         print $self->foo, "\n";
     });
+    $obj->say_foo;
 
-    done_testing();
+Inheritance with the prototype chain:
+
+    my $proto = JIP::Object->new;
+
+    # Creates an own method 'as_string' on $proto
+    $proto->method('as_string', sub { 'prototype' });
+
+    my $obj = JIP::Object->new(proto => $proto);
+
+    ref $obj->own_method('as_string'); # q{}
+    $obj->as_string;                   # q{prototype}
+
+    # Creates an own method 'as_string' on $obj
+    $obj->method('as_string', sub { 'object' });
+
+    # If the method is found in $obj, then proto is not checked.
+    ref $obj->own_method('as_string'); # q{CODE}
+    $obj->as_string;                   # q{object}
+
+    # The end of the prototype chain as undef.
+    ref $obj->proto->proto; # q{}
 
 =head1 METHODS
 
 =head2 new
 
+Construct a new L<JIP::Object> object.
+
     my $obj = JIP::Object->new;
     my $obj = JIP::Object->new(proto => JIP::Object->new);;
 
-Construct a new L<JIP::Object> object.
-
 =head2 has
 
-    $obj = $obj->has('x', get => 'x', set => 'set_x');
-
 Define a new property.
 
+    # Public
+    $obj->has('foo', (get => '+', set => '+'));
+    $obj->set_foo(42)->foo; # 42
+
+    # Private
+    $obj->has('bar', (get => '-', set => '-'));
+    $obj->_set_bar(42)->_bar; # 42
+
+    # Create user-defined names for getters/setters
+    $obj->has('wtf' => (get => 'wtf_getter', set => 'wtf_setter'));
+    $obj->wtf_setter(42)->wtf_getter; # 42
+
+    # Pass an optional first argument of setter to set
+    # a default value, it should be a constant or callback.
+    $obj->has('baz' => (get => '+', set => '+', default => 42));
+    ($self->set_baz->baz; # 42
+
+    $obj->has('qux' => (get => '+', set => '+', default => sub {
+        my $self = shift;
+        return $self->baz;
+    }));
+    $self->set_qux->qux; # 42
+
+    # Declaring multiple attributes in a single declaration
+    $obj->has([qw(one two three)]);
+
 =head2 method
 
+Define a new method.
+
     $obj = $obj->method('get_x', sub {
         my $self = shift;
         return $self->x;
     });
 
-Define a new method.
-
 =head2 proto
 
-A getter function.
+Returns the value of the 'proto' attribute.
+
+    $obj->proto;
 
 =head2 set_proto
 
-A setter function.
+Sets the value of the 'proto' attribute.
+
+    $obj->set_proto($proto);
 
 =head2 own_method
 
